@@ -289,89 +289,113 @@ GQuark tp_get_conn_renaming_interface()
   return ret;
 }
 
-TpConn *tp_conn_new(DBusGConnection *connection,
-		    const gchar      *bus_name,
-		    const gchar      *object_path)
 
+TpConn *
+tp_conn_new_without_connect (DBusGConnection *connection,
+                             const gchar *bus_name,
+                             const gchar *object_path,
+                             guint *status,
+                             GError **error)
 {
-  GError *error = NULL;
   gchar *unique_name;
   gchar **interfaces;
   guint conn_status = TP_CONN_STATUS_DISCONNECTED;
   TpConn *obj;
 
-  g_return_val_if_fail(connection != NULL, NULL);
-  g_return_val_if_fail(bus_name != NULL, NULL);
-  g_return_val_if_fail(object_path, NULL);
+  g_return_val_if_fail (connection != NULL, NULL);
+  g_return_val_if_fail (bus_name != NULL, NULL);
+  g_return_val_if_fail (object_path, NULL);
 
   /* Create the proxy object for this connection. It will be used to
      perform the actual method calls over D-BUS. */
 
-  if (!dbus_g_proxy_call (tp_get_bus_proxy (), 
-                          "GetNameOwner", &error, G_TYPE_STRING, bus_name, 
-                          G_TYPE_INVALID, G_TYPE_STRING, &unique_name, 
-                          G_TYPE_INVALID))
-  {
-    g_warning("tp_conn_new: getting unique name failed: %s\n", error->message);
-    g_error_free(error);
-    return NULL;
-  }
-
-  obj = g_object_new(TELEPATHY_CONN_TYPE,
-                             "name", unique_name,
-                             "path", object_path,
-                             "interface", TP_IFACE_CONN_INTERFACE,
-                             "connection", connection, NULL);
-  g_free(unique_name);
-
-  g_datalist_init(&(obj->interface_list));
-
-  dbus_g_proxy_add_signal(DBUS_G_PROXY(obj), "NewChannel",
-                          DBUS_TYPE_G_OBJECT_PATH, G_TYPE_STRING,
-                          G_TYPE_UINT, G_TYPE_UINT, G_TYPE_BOOLEAN,
-                          G_TYPE_INVALID);
-  dbus_g_proxy_add_signal(DBUS_G_PROXY(obj), "StatusChanged",
-                          G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID);
-
-  /* Check if the connection is already connected. If yes, we can
-     already perform GetInterfaces(). */ 
-
-  if (!tp_conn_get_status(DBUS_G_PROXY(obj), &conn_status, &error))
-  {
-    g_warning("GetStatus() failed: %s\n", error->message);
-    g_error_free(error);
-  }
-  
-  if (conn_status == TP_CONN_STATUS_CONNECTED)
-  {
-    if (!tp_conn_get_interfaces(DBUS_G_PROXY(obj), &interfaces, &error))
+  if (!dbus_g_proxy_call (tp_get_bus_proxy (), "GetNameOwner", error,
+        G_TYPE_STRING, bus_name, G_TYPE_INVALID,
+        G_TYPE_STRING, &unique_name, G_TYPE_INVALID))
     {
-      _tp_warn_failure ("GetInterfaces()", error);
-      g_error_free(error);
-      return obj;
-    } 
+      return NULL;
+    }
 
-    /* Initialize the interface objects for this TpConn object */
-    tp_conn_local_set_interfaces(obj, interfaces);
+  obj = g_object_new (TELEPATHY_CONN_TYPE,
+      "name", unique_name,
+      "path", object_path,
+      "interface", TP_IFACE_CONN_INTERFACE,
+      "connection", connection,
+      NULL);
+  g_free (unique_name);
 
-    g_strfreev(interfaces);
+  g_datalist_init (&(obj->interface_list));
 
-    return obj;
-  }
+  dbus_g_proxy_add_signal (DBUS_G_PROXY (obj), "NewChannel",
+      DBUS_TYPE_G_OBJECT_PATH, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT,
+      G_TYPE_BOOLEAN, G_TYPE_INVALID);
+  dbus_g_proxy_add_signal(DBUS_G_PROXY (obj), "StatusChanged",
+      G_TYPE_UINT, G_TYPE_UINT, G_TYPE_INVALID);
+
+  /* Check if the connection is already connected. If so, we can
+     already perform GetInterfaces(). */
+
+  if (!tp_conn_get_status (DBUS_G_PROXY (obj), &conn_status, error))
+    {
+      g_object_unref (obj);
+      return NULL;
+    }
+
+  if (conn_status == TP_CONN_STATUS_CONNECTED)
+    {
+      if (!tp_conn_get_interfaces (DBUS_G_PROXY (obj), &interfaces, error))
+        {
+          g_object_unref (obj);
+          return NULL;
+        }
+
+      /* Initialize the interface objects for this TpConn object */
+      tp_conn_local_set_interfaces (obj, interfaces);
+
+      g_strfreev (interfaces);
+    }
   else
     {
-
-  /* Not connected yet, so this is really a new connection. Thus, we
-     have to hook up to StatusChanged signal to perform the
-     GetInterfaces eventually there and then call Connect() to initiate the
-     connection setup */
+      /* Not connected yet, so this is really a new connection. Thus, we
+         have to hook up to StatusChanged signal to perform the
+         GetInterfaces when it goes Connected */
 
       dbus_g_proxy_connect_signal(DBUS_G_PROXY(obj), "StatusChanged",
                                   G_CALLBACK(tp_conn_status_change_handler),
                                   NULL, NULL);
+    }
 
-      tp_conn_connect_async(DBUS_G_PROXY(obj),
-                            tp_conn_connect_req_handler, NULL);
+  if (status != NULL)
+    *status = conn_status;
+  return obj;
+}
+
+
+TpConn *
+tp_conn_new (DBusGConnection *connection,
+             const gchar *bus_name,
+             const gchar *object_path)
+{
+  GError *error = NULL;
+  guint status;
+  TpConn *obj;
+
+  obj = tp_conn_new_without_connect (connection, bus_name, object_path,
+      &status, &error);
+
+  if (obj == NULL)
+    {
+      /* either GetNameOwner, GetStatus or GetInterfaces failed */
+      g_warning ("Failed to create connection for %s %s: %s", bus_name,
+          object_path, error->message);
+      g_error_free (error);
+      return NULL;
+    }
+
+  if (status != TP_CONN_STATUS_CONNECTED)
+    {
+      tp_conn_connect_async (DBUS_G_PROXY(obj),
+                             tp_conn_connect_req_handler, NULL);
     }
 
   return obj;
